## example of binomial categorical data
## Pristiq nausea data

modelName <- "binomialExample" # root names of model file

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") ## change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) # not required but assures repeatable results

################################################################################################
# data: incidence of nausea reported in Pristiq package insert

xdata = data.frame(
	percentNausea = c(10,22,26,36,41),
	dose = c(0,50,100,200,400),
	nTotal = c(636,317,424,307,317)) 
xdata$nNausea = round(xdata$percentNausea*xdata$nTotal/100)

# create WinBUGS data set
bugsdata = list(
	nobs=nrow(xdata),
	dose=xdata$dose,
	nNausea=xdata$nNausea,
	nTotal=xdata$nTotal
)

# create initial estimates
bugsinit = function() list(
	alpha = rnorm(1,logit(0.1),0.5),
	beta = rnorm(1,0.01,0.5),
	gamma = runif(1,0.25,2)
)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("alpha","beta","gamma")

## Additional variables to monitor
otherRVs <- c("nNauseaPred","pNausea")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
## run WinBUGS

n.chains = 3
n.iter = 100000
n.burnin = 10000
n.thin = 50
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = TRUE, debug = FALSE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory

save.model(bugs.fit, modelName)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
## posterior predictive distributions

## prediction of future observations in the same subjects, i.e., posterior predictions
## conditioned on observed data from the same subject

pred = posterior[,grep("nNauseaPred\\[",dimnames(posterior)[[2]])]

x1 = xdata
x1$dose = as.numeric(as.character(x1$dose))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$nNausea = as.vector(t(apply(pred,2,quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

print(xyplot(I(percentNausea/100)~dose,xdata,
	type="b",lty=1,col="blue",
	pch=19,lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose (mg/d)",cex=1.2),
	ylab=list(label="fraction of patients with nausea",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1)),
	position=c(0,0,0.5,1),more=T)

print(xyplot(logit(percentNausea/100)~dose,xdata,
	type="b",lty=1,col="blue",
	pch=19,lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose (mg/d)",cex=1.2),
	ylab=list(label="logit(fraction of patients with nausea)",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1)),
	position=c(0.5,0,1,1),more=F)

print(xyplot(I(nNausea/nTotal)~dose,x1,groups=type,
	panel=panel.superpose,type=c("l","l","l","p"),lty=c(3,3,1,0),col=c(2,2,4,1),
	pch=c(NA,NA,NA,19),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose (mg/d)",cex=1.2),
	ylab=list(label="fraction of patients with nausea",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1)),
	position=c(0,0,0.5,1),more=T)

print(xyplot(logit(I(nNausea/nTotal))~dose,x1,groups=type,
	panel=panel.superpose,type=c("l","l","l","p"),lty=c(3,3,1,0),col=c(2,2,4,1),
	pch=c(NA,NA,NA,19),lwd=2,cex=1,scales=list(cex=1),
		xlab=list(label="dose (mg/d)",cex=1.2),
	ylab=list(label="logit(fraction of patients with nausea)",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1)),
	position=c(0.5,0,1,1),more=F)

dev.off()
