## example of binomial categorical data with mixed effects
## VTE and bleeding data for factor Xa inhibtors

modelName <- "binomialMEExample" # root names of model file

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") ## change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
library(locfit)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## get data file
fxa.clin.data = read.csv("Fxa metadata.csv",as.is=T)
fxa.clin.data$rate = fxa.clin.data$events/fxa.clin.data$n

## create WinBUGS data set

xdata = fxa.clin.data[fxa.clin.data$event.type=="vte",]
xdata = xdata[c("study","drug","daily.dose","surgery","events","n")]

## combine knee and hip outcomes for study 5 (the only one with both types of patients)
x1 = xdata[xdata$study==5&xdata$surgery=="thr",]
x2 = xdata[xdata$study==5&xdata$surgery=="tkr",]
x2 = x2[c("daily.dose","drug","events","n")]
names(x2)[names(x2)=="events"] = "events2"
names(x2)[names(x2)=="n"] = "n2"
x1 = merge(x1,x2)
x1$events = x1$events + x1$events2
x1$n = x1$n + x1$n2
x1$events2 = NULL
x1$n2 = NULL

x2 = xdata[xdata$study==5&xdata$surgery=="thr",]
x2$events = NULL
x2$n = NULL
x2 = merge(x2,x1)
x2$surgery = rep("both",nrow(x2))

xdata = xdata[xdata$study!=5,]
xdata = rbind(xdata,x2)
xdata = xdata[order(xdata$study,xdata$drug,xdata$daily.dose),]

################################################################################################

## create WinBUGS data set
bugsdata = list(
	nobs=nrow(xdata),
	nstudy=length(unique(xdata$study)),
	study=xdata$study,
	drug=as.numeric(swap(xdata$drug,
		c("enoxaparin","rivaroxaban","apixaban","LY517717","razaxaban","YM150"),1:6)),
	dose=xdata$daily.dose,
	vte=xdata$events,
	n=xdata$n)

## create initial estimates
bugsinit = function() list(
	ep.hat = logit(0.45),
	log.sigma = 0.5,
	theta=rnorm(6,c(-0.05,-0.05,-0.05,-0.05,-0.05,-0.05),0.1)
)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("ep.hat","theta","sigma")

## Additional variables to monitor
otherRVs <- c("ep","p.vte","vte.cond","vte.pred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
## run WinBUGS

n.chains = 3
n.iter = 10000
n.burnin = 4000
n.thin = 5
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = TRUE, debug = FALSE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory

save.model(bugs.fit, modelName)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
## posterior predictive distributions

## prediction of future observations in the same studies, i.e., posterior predictions
## conditioned on observed data from the same study

pred = posterior[,grep("vte.cond\\[",dimnames(posterior)[[2]])]

x1 = xdata
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$events = as.vector(t(apply(pred,2,quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)
x1$rate = x1$events/x1$n

xyplot(rate~daily.dose|paste(drug,"study",study),x1[x1$drug!="enoxaparin",],groups=type,
	panel=function(x,y,subscripts,groups,...){
		study = x1$study[x1$drug!="enoxaparin"][subscripts[1]]
		enox = x1[x1$drug=="enoxaparin"&x1$study==study,]
		panel.abline(h=enox$rate[enox$type=="observed"],lwd=2,lty=1,col="grey")
		panel.abline(h=enox$rate[enox$type=="median"],lwd=2,lty=3,col="blue")
		panel.abline(h=enox$rate[enox$type=="5%ile"|enox$type=="95%ile"],lwd=2,lty=3,col="red")
		panel.superpose(x,y,subscripts,groups,...)
	},
	type=c("l","l","l","p"),lty=c(1,1,1,0),col=c(2,2,4,1),
	pch=c(NA,NA,NA,19),lwd=2,cex=1,scales=list(x=list(relation="free"),cex=1),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with VTE",cex=1.2),
	par.strip.text=list(cex=0.8))

x1 = x1[x1$type=="observed",]
xyplot(rate~daily.dose|paste(drug,"study",study),x1[x1$drug!="enoxaparin",],
	panel=function(x,y,subscripts,groups,...){
		study = x1$study[x1$drug!="enoxaparin"][subscripts[1]]
		enox = x1[x1$drug=="enoxaparin"&x1$study==study,]
		panel.abline(h=enox$rate[enox$type=="observed"],lwd=2,lty=1,col="red")
		panel.xyplot(x,y,...)
	},
	type="b",lty=c(1),col="blue",
	pch=19,lwd=2,cex=1,scales=list(x=list(relation="free"),cex=1),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with VTE",cex=1.2),
	par.strip.text=list(cex=0.8))


## prediction of future observations in new studies of the same design, i.e., posterior predictions

pred = posterior[,grep("vte.pred\\[",dimnames(posterior)[[2]])]

x1 = xdata
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$events = as.vector(t(apply(pred,2,quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)
x1$rate = x1$events/x1$n

xyplot(rate~daily.dose|paste(drug,"study",study),x1[x1$drug!="enoxaparin",],groups=type,
	panel=function(x,y,subscripts,groups,...){
		study = x1$study[x1$drug!="enoxaparin"][subscripts[1]]
		enox = x1[x1$drug=="enoxaparin"&x1$study==study,]
		panel.abline(h=enox$rate[enox$type=="observed"],lwd=2,lty=1,col="grey")
		panel.abline(h=enox$rate[enox$type=="median"],lwd=2,lty=3,col="blue")
		panel.abline(h=enox$rate[enox$type=="5%ile"|enox$type=="95%ile"],lwd=2,lty=3,col="red")
		panel.superpose(x,y,subscripts,groups,...)
	},
	type=c("l","l","l","p"),lty=c(1,1,1,0),col=c(2,2,4,1),
	pch=c(NA,NA,NA,19),lwd=2,cex=1,scales=list(x=list(relation="free"),cex=1),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with VTE",cex=1.2),
	par.strip.text=list(cex=0.8))

dev.off()

