## example of time-to-event data modeled with a piecewise constant hazard function
## fictional dropout data

modelName <- "TTEExample2" # root names of model file

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") ## change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## get data file
dataTTE <- read.csv("PANSSData.csv",as.is=T)

dataTTE$tEnd <- as.vector(
  sapply(unique(dataTTE$patient), 
         function(patient, data)
         c(data$time[data$patient==patient][-1], NA),
         data = dataTTE))

dataTTE <- dataTTE[!is.na(dataTTE$PANSS) & !is.na(dataTTE$tEnd), ]
dataTTE$censored <- dataTTE$tDrop >= dataTTE$tEnd
dataTTE$tEnd <- ifelse(dataTTE$tDrop < dataTTE$tEnd, dataTTE$tDrop, dataTTE$tEnd)

################################################################################################

## create WinBUGS data set

bugsdata <- list(
  nObs = nrow(dataTTE),
  PANSS = dataTTE$PANSS,
  PANSS0 = dataTTE$PANSS0,
  tDrop = ifelse(dataTTE$censored, NA, dataTTE$tEnd - dataTTE$time),
  tCensor = ifelse(!dataTTE$censored, 0, dataTTE$tEnd - dataTTE$time)
  )

## create initial estimates
bugsinit = function() list(
		logH0 = rnorm(1, log(-log(0.4)/6), 0.4),
		beta = rnorm(1, 0.1, 0.05)
	)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("h0", "beta")

## Additional variables to monitor
otherRVs <- c("tDropPred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
# run WinBUGS

n.chains = 3
n.iter = 50000
n.burnin = 10000
n.thin = 25
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = FALSE, debug = FALSE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory

save.model(bugs.fit, modelName, n.chains = n.chains)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
## posterior predictive distributions

doses = sort(unique(dataTTE$dose))

pred = posterior[,grep("tDropPred\\[",dimnames(posterior)[[2]])]

x1 = dataTTE[!duplicated(dataTTE$patient),]
nPred = 120
tDropPred = seq(0, 6, len=nPred+1)[-(nPred+1)]
tDrop = data.frame(dose = rep(doses, ea = nPred), 
	tDrop = rep(tDropPred, length(unique(x1$dose))),
	fNoDrop = do.call(c, tapply(x1$tDrop, list(dose=x1$dose), 
		function(x,xout){
			cdfFun = ecdf(x)
			1 - cdfFun(xout)
		},
		xout = tDropPred))
)
tDrop$type = rep("observed", nrow(tDrop))

x1 = rbind(tDrop, tDrop, tDrop)
x1$fNoDrop = as.vector(t(apply(
	apply(pred, 1,
		function(x, xout, dose){
			do.call(c, tapply(x, list(dose=dose), 
				function(x,xout){
					cdfFun = ecdf(x)
					1 - cdfFun(xout)
				},
				xout = xout))
		},
		xout = tDropPred, dose = dataTTE$dose),
	1, quantile, probs=c(0.05,0.5,0.95))))
x1$type = rep(c("5%ile","median","95%ile"), ea=nrow(tDrop))
tDrop = rbind(tDrop, x1)

xyplot(fNoDrop ~ tDrop | as.factor(dose), tDrop, groups = type,
	type="s",lty=c(2,2,1,1),
	col=c("blue","blue","red","black"),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="fraction of patients remaining in trial",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

dev.off()	
