## example of binary categorical data
## fictional pulmonary exacerbation data

modelName <- "handsOn1" # root names of model file

if(.Platform$OS.type == "windows")
  Sys.setenv(HOME = substr(R.home(), 1, 2))

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255Jags")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup

setwd(exampleDir)
library(rjags)
library(coda)
library(lattice)
library(locfit)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## get data files
dataProb1 = read.csv("ME2ExacerbationData.csv", as.is=T)

################################################################################################
## exploratory plots

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"EdaPlots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

x1 = aggregate(dataProb1["exac"],list(dose=dataProb1$dose,AntiBio=dataProb1$AntiBio),sum)
x1$n = as.vector(tapply(dataProb1$exac,list(dose=dataProb1$dose,AntiBio=dataProb1$AntiBio),length))
x1$mean = x1$exac/x1$n
x2 = sapply(1:nrow(x1),function(i,y) binom.test(y$exac[i],y$n[i])$conf.int,y=x1)
x1$lci = x2[1,]
x1$uci = x2[2,]
xyplot(mean~dose,data=x1,groups=AntiBio,
       ly=x1$lci,uy=x1$uci,
       prepanel=prepanel.ci,panel=panel.superpose,panel.groups=panel.ci,
       type="b",
       auto.key=list(text=c("no chronic antibiotic","chronic antibiotic"),cex=1.2),
       cex=1.2,scales=list(cex=1.2),
       xlab=list(label="dose",cex=1.2),
       ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

xyplot(logit(mean)~dose,data=x1,groups=AntiBio,
	type="b",
	auto.key=list(text=c("no chronic antibiotic","chronic antibiotic"),cex=1.2),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="logit(fraction of patients with pulmonary exacerbation)",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

x1 = aggregate(dataProb1["exac"],list(dose=dataProb1$dose,DNase=dataProb1$DNase),sum)
x1$n = as.vector(tapply(dataProb1$exac,list(dose=dataProb1$dose,DNase=dataProb1$DNase),length))
x1$mean = x1$exac/x1$n
x2 = sapply(1:nrow(x1),function(i,y) binom.test(y$exac[i],y$n[i])$conf.int,y=x1)
x1$lci = x2[1,]
x1$uci = x2[2,]
xyplot(mean~dose,data=x1,groups=DNase,
	ly=x1$lci,uy=x1$uci,
	prepanel=prepanel.ci,panel=panel.superpose,panel.groups=panel.ci,
	type="b",
	auto.key=list(text=c("no rhDNase","rhDNase"),cex=1.2),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

xyplot(logit(mean)~dose,data=x1,groups= DNase,
	type="b",
	auto.key=list(text=c("no chronic antibiotic","chronic antibiotic"),cex=1.2),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="logit(fraction of patients with pulmonary exacerbation)",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

fit = locfit(exac~FEV1Baseline+as.factor(dose),dataProb1,link="logit")
x1 = scb(fit,ev=lfgrid(c(50,4)),link="logit",simul=F)
pred = rbind(data.frame(FEV1Baseline=x1$xev[,1],dose=x1$xev[,2],exac=logit.inv(x1$coef)),
	data.frame(FEV1Baseline=x1$xev[,1],dose=x1$xev[,2],exac=logit.inv(x1$lower)),
	data.frame(FEV1Baseline=x1$xev[,1],dose=x1$xev[,2],exac=logit.inv(x1$upper)))
pred$type = rep(c("mean","lower","upper"),ea=length(x1$coef))
x1 = dataProb1[c("dose","FEV1Baseline","exac")]
x1$type =rep("observed",nrow(x1))
pred=rbind(x1,pred)

xyplot(exac~FEV1Baseline|as.factor(dose),pred,groups=type,
	panel=panel.superpose,type=c("l","l","p","l"),lty=c(3,1,0,3),
	col=c("red","blue","black","red"),
	pch=c(NA,NA,19,NA),lwd=2,
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="baseline FEV1",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

fit = locfit(exac~age+as.factor(dose),dataProb1,link="logit")
x1 = scb(fit,ev=lfgrid(c(50,4)),link="logit",simul=F)
pred = rbind(data.frame(age=x1$xev[,1],dose=x1$xev[,2],exac=logit.inv(x1$coef)),
	data.frame(age=x1$xev[,1],dose=x1$xev[,2],exac=logit.inv(x1$lower)),
	data.frame(age=x1$xev[,1],dose=x1$xev[,2],exac=logit.inv(x1$upper)))
pred$type = rep(c("mean","lower","upper"),ea=length(x1$coef))
x1 = dataProb1[c("dose","age","exac")]
x1$type =rep("observed",nrow(x1))
pred=rbind(x1,pred)

xyplot(exac~age|as.factor(dose),pred,groups=type,
	panel=panel.superpose,type=c("l","l","p","l"),lty=c(3,1,0,3),
	col=c("red","blue","black","red"),
	pch=c(NA,NA,19,NA),lwd=2,
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="age (y)",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

xyplot(FEV1Baseline~age,dataProb1,
	panel=function(x,y,...){
		panel.xyplot(x,y,...)
		panel.loess(x,y,lwd=2,col="black",...)
	},
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="age (y)",cex=1.2),
	ylab=list(label="baseline FEV1",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

dev.off()

################################################################################################

## create WinBUGS data set
bugsdata = list(
	nobs=nrow(dataProb1),
	dose=dataProb1$dose,
	FEV1Baseline=dataProb1$FEV1Baseline,
	AntiBio=dataProb1$AntiBio,
	DNase=dataProb1$DNase,
	age=dataProb1$age,
	exac=dataProb1$exac
)

glmFit = glm(exac~dose+I(FEV1Baseline-70)+AntiBio+DNase+I(age-18),data=dataProb1,family=binomial)

## create initial estimates
bugsinit = function() list(
	theta=rnorm(6,coef(glmFit),0.6)
)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("theta")

## Additional variables to monitor
otherRVs <- c("pExac","exacPred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
## run WinBUGS

n.chains <- 3
n.iter <- 10000
n.burnin <- 5000
n.adapt <- 1000
n.thin <- 5

jagsModel <- jags.model(file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
           data = bugsdata,
           inits = bugsinit,
           n.chains = n.chains,
           n.adapt = n.adapt)

update(jagsModel, n.iter = n.burnin - n.adapt)

fit <- coda.samples(model = jagsModel,
                           variable.names = parameters, n.iter = n.iter - n.burnin, thin = n.thin)

## save scripts, data and results to a directory

save.model(fit, modelName)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- aperm(as.array(fit), c(1,3,2))
posterior <- as.matrix(fit)

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
## posterior predictive distributions

## prediction of future observations in the same subjects, i.e., posterior predictions of the
## observed" fraction of patients with >=1 pulmonary exacerbation assuming the same number of
## patients with the same covariates

## Get predicted data
pred = posterior[,grep("exacPred\\[",dimnames(posterior)[[2]])]

## Calculate fraction of patients with >=1 pulmonary exacerbation in each treatment
## arm and antibiotic use subset.
x1 = aggregate(dataProb1["exac"],list(dose=dataProb1$dose,AntiBio=dataProb1$AntiBio),mean)
x1$dose = as.numeric(as.character(x1$dose))
x1$type =rep("observed",nrow(x1))

## Do the same for each MCMC sample and calculate the posterior medians and 90% credible
## intervals.
x2 = rbind(x1,x1,x1)
x2$exac = as.vector(t(sapply(tapply(1:nrow(dataProb1),
	list(dose=dataProb1$dose,AntiBio=dataProb1$AntiBio),
	function(i,pred) apply(pred[,i],1,mean),pred=pred),quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

## Plot fraction of patients with >=1 pulmonary exacerbation vs dose showing posterior medians
## and 90% credible intervals along with the observaed data.

## All on one panel
xyplot(exac~dose,x1,groups=paste(type,AntiBio),
	panel=panel.superpose,type=c(rep("l",6),rep("p",2)),lty=c(rep(2,4),rep(1,2),rep(0,2)),
	col=c("blue","red"),pch=c(rep(NA,6),rep(19,2)),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	ylim=c(0,1.1*max(x1$exac)),
	key=list(x=0.4,y=0.8,text=list(labels=c("no chronic antibiotic","chronic antibiotic"),
	cex=1.2),lines=list(type="b",col=c("blue","red"),pch=19,lwd=2,cex=1)))

## Separate panels for patients with and without chronic antibiotic use
xyplot(exac~dose|factor(AntiBio,labels=c("no chronic antibiotic","chronic antibiotic")),x1,
	groups=type,panel=panel.superpose,type=c("l","l","l","p"),lty=c(2,2,1,0),col=c(2,2,4,1),
	pch=c(NA,NA,NA,19),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	ylim=c(0,1.1*max(x1$exac)),
	strip = function(...) strip.default(..., style = 1))

## Calculate fraction of patients with >=1 pulmonary exacerbation in each treatment
## arm and antibiotic use subset.
x1 = aggregate(dataProb1["exac"],list(dose=dataProb1$dose,AntiBio=dataProb1$AntiBio),mean)
x1$dose = as.numeric(as.character(x1$dose))

## Do the same for each MCMC sample. 
x2 = tapply(1:nrow(dataProb1),
	list(dose=dataProb1$dose,AntiBio=dataProb1$AntiBio),
	function(i,pred) apply(pred[,i],1,mean),pred=pred)
x2 = cbind(expand.grid(dimnames(x2))[rep(1:prod(dim(x2)), ea = nrow(pred)),],
	exac = do.call(c, x2))
x2$dose = as.numeric(as.character(x2$dose))

## Plot posterior density of the predicted fraction of patients with >=1 pulmonary exacerbation
## for each dose and antibiotic use group. On each panel overlay a vertical line showing the
## observed value. Also calculate the fraction of predicted values >= the observed value and
## add text showing that value.
densityplot(~exac | factor(AntiBio,labels=c("no chronic antibiotic","chronic antibiotic"))*
	as.factor(dose), x2,
	subscripts=T,
	panel=function(x,y,subscripts,...){
		panel.densityplot(x,...)
		dose = x2$dose[subscripts[1]]
		AntiBio = x2$AntiBio[subscripts[1]]
		exacObs = x1$exac[x1$dose==dose & x1$AntiBio==AntiBio]
		panel.abline(v=exacObs,col="red",lwd=2,lty=2)
		ltext(x=min(x)+0.99*(max(x)-min(x)),y=3,adj=c(1,0),
			labels=paste("Pr(pred >= obs) =",signif(sum(x>=exacObs)/length(x),3),sep=""))
	},
	xlab=list(label="fraction of patients with pulmonary exacerbation",cex=1.25),
	ylab=list(cex=1.25),
	scales=list(x=list(relation="free"),cex=1.25),
	par.strip.text=list(cex=1))

## Do it all over again for data subsetted by rhDNase use.

x1 = aggregate(dataProb1["exac"],list(dose=dataProb1$dose,DNase=dataProb1$DNase),mean)
x1$dose = as.numeric(as.character(x1$dose))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$exac = as.vector(t(sapply(tapply(1:nrow(dataProb1),list(dose=dataProb1$dose,
	Nase=dataProb1$DNase),function(i,pred) apply(pred[,i],1,mean),pred=pred),
	quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(exac~dose,x1,groups=paste(type,DNase),
	panel=panel.superpose,type=c(rep("l",6),rep("p",2)),lty=c(rep(2,4),rep(1,2),rep(0,2)),
	col=c("blue","red"),pch=c(rep(NA,6),rep(19,2)),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	ylim=c(0,1.1*max(x1$exac)),
	key=list(x=0.4,y=0.8,text=list(labels=c("no rhDNase","rhDNase"),cex=1.2),
	lines=list(type="b",col=c("blue","red"),pch=19,lwd=2,cex=1)))

xyplot(exac~dose|factor(DNase,labels=c("no rhDNase","rhDNase")),x1,groups=type,
	panel=panel.superpose,type=c("l","l","l","p"),lty=c(2,2,1,0),col=c(2,2,4,1),
	pch=c(NA,NA,NA,19),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	ylim=c(0,1.1*max(x1$exac)),
	strip = function(...) strip.default(..., style = 1))

################################################################################################
## Plots of simulations to show model-predicted effects of dose and patient-specific covariates

theta = posterior[,grep("theta\\[",dimnames(posterior)[[2]])]

calcExac = function(dose,FEV1Baseline,AntiBio,DNase,age,theta)
	logit.inv(theta[1] + theta[2]*dose + theta[3]*(FEV1Baseline-70) + theta[4]*AntiBio +
		theta[5]*DNase + theta[6]*(age-18))

## Pr(>= 1 exacerbation) by dose, chronic antibiotic use and rhDNase use for 18 year old CF
## patients with typical baseline FEV1 (70% of predicted)

doses = seq(0,max(dataProb1$dose),len=21)
x1 = expand.grid(dose=doses,AntiBio=c(0,1),FEV1Baseline=70,DNase=c(0,1),age=18)
pred = rbind(x1,x1,x1)
pred$exac = as.vector(t(apply(sapply(1:nrow(theta),
	function(i,dose,FEV1Baseline,AntiBio,DNase,age,theta)
	calcExac(dose,FEV1Baseline,AntiBio,DNase,age,theta[i,]),
	dose=x1$dose,FEV1Baseline=x1$FEV1Baseline,AntiBio=x1$AntiBio,DNase=x1$DNase,
	age=x1$age,theta=theta),1,
	quantile,probs=c(0.05,0.5,0.95))))
pred$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))

xyplot(exac~dose|factor(DNase,labels=c("no rhDNase","rhDNase")),pred,groups=paste(type,AntiBio),
	panel=panel.superpose,type=c(rep("l",6),rep("p",2)),lty=c(rep(2,4),rep(1,2),rep(0,2)),
	col=c("blue","red"),pch=c(rep(NA,6),rep(19,2)),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	ylim=c(0,1.1*max(pred$exac)),
	key=list(x=0.4,y=0.8,
	text=list(labels=c("no chronic antibiotic","chronic antibiotic"),cex=1.2),
	lines=list(type="b",col=c("blue","red"),pch=19,lwd=2,cex=1)))

xyplot(exac~dose|factor(AntiBio,labels=c("no chronic antibiotic","chronic antibiotic")),pred,
	groups=paste(type,DNase),
	panel=panel.superpose,type=c(rep("l",6),rep("p",2)),lty=c(rep(2,4),rep(1,2),rep(0,2)),
	col=c("blue","red"),pch=c(rep(NA,6),rep(19,2)),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	ylim=c(0,1.1*max(pred$exac)),
	key=list(x=0.4,y=0.8,text=list(labels=c("no rhDNase","rhDNase"),cex=1.2),
	lines=list(type="b",col=c("blue","red"),pch=19,lwd=2,cex=1)))

## Pr(>= 1 exacerbation) by baseline FEV1 , dose, chronic antibiotic use and rhDNase use for 
## year old CF patients

doses = sort(unique(dataProb1$dose))
FEV1Baseline = seq(min(dataProb1$FEV1Baseline),max(dataProb1$FEV1Baseline),len=21)
x1 = expand.grid(dose=doses,AntiBio=c(0,1),FEV1Baseline=FEV1Baseline,DNase=c(0,1),age=18)
pred = rbind(x1,x1,x1)
pred$exac = as.vector(t(apply(sapply(1:nrow(theta),
	function(i,dose,FEV1Baseline,AntiBio,DNase,age,theta)
	calcExac(dose,FEV1Baseline,AntiBio,DNase,age,theta[i,]),
	dose=x1$dose,FEV1Baseline=x1$FEV1Baseline,AntiBio=x1$AntiBio,
	DNase=x1$DNase,age=x1$age,theta=theta),1,
	quantile,probs=c(0.05,0.5,0.95))))
pred$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))

xyplot(exac~FEV1Baseline|as.factor(dose)*factor(DNase,labels=c("no rhDNase","rhDNase")),
	pred,groups=paste(type,AntiBio),
	panel=panel.superpose,type=c(rep("l",6),rep("p",2)),lty=c(rep(2,4),rep(1,2),rep(0,2)),
	col=c("blue","red"),pch=c(rep(NA,6),rep(19,2)),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="baseline FEV1 % of predicted",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	ylim=c(0,1.1*max(pred$exac)),
	key=list(x=0.4,y=0.8,
	text=list(labels=c("no chronic antibiotic","chronic antibiotic"),cex=1.2),
	lines=list(type="b",col=c("blue","red"),pch=19,lwd=2,cex=1)))

## Pr(>= 1 exacerbation) by age, dose, chronic antibiotic use and rhDNase use for CF
## patients with baseline FEV1 = 70% of predicted

doses = sort(unique(dataProb1$dose))
age = seq(min(dataProb1$age),max(dataProb1$age),len=21)
x1 = expand.grid(dose=doses,AntiBio=c(0,1),FEV1Baseline=70,DNase=c(0,1),age=age)
pred = rbind(x1,x1,x1)
pred$exac = as.vector(t(apply(sapply(1:nrow(theta),
	function(i,dose,FEV1Baseline,AntiBio,DNase,age,theta)
	calcExac(dose,FEV1Baseline,AntiBio,DNase,age,theta[i,]),
	dose=x1$dose,FEV1Baseline=x1$FEV1Baseline,AntiBio=x1$AntiBio,DNase=x1$DNase,
	age=x1$age,theta=theta),1,
	quantile,probs=c(0.05,0.5,0.95))))
pred$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))

xyplot(exac~age|as.factor(dose)*factor(DNase,labels=c("no rhDNase","rhDNase")),
	pred,groups=paste(type,AntiBio),
	panel=panel.superpose,type=c(rep("l",6),rep("p",2)),lty=c(rep(2,4),rep(1,2),rep(0,2)),
	col=c("blue","red"),pch=c(rep(NA,6),rep(19,2)),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="age",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	ylim=c(0,1.1*max(pred$exac)),
	key=list(x=0.4,y=0.8,
	text=list(labels=c("no chronic antibiotic","chronic antibiotic"),cex=1.2),
	lines=list(type="b",col=c("blue","red"),pch=19,lwd=2,cex=1)))

################################################################################################
## residual analysis

## posterior distributions of standard residuals

## Get predicted Pr(>= 1 pulmonary exacerbation)
pred = posterior[,grep("pExac\\[",dimnames(posterior)[[2]])]

## Get the observed data
x1 = dataProb1
## Calculate the posterior mean of Pr(>= 1 pulmonary exacerbation)
x1$pExacExp = apply(pred, 2, mean)

## Calculate the posterior medians and 90% credible intervals of the residuals
residual = t(sapply(1:nrow(x1), function(i, data, pred) quantile(x1$exac[i] - pred[,i],
	probs=c(0.05,0.5,0.95)), data = x1, pred = pred))
dimnames(residual)[[2]] = c("residLCI", "residMed", "residUCI")

x1 = cbind(x1, residual)

## Plot the residuals versus the mean predicted Pr(>= 1 pulmonary exacerbation)
xyplot(residMed ~ pExacExp | as.factor(dose), data=x1, groups=AntiBio,
	ly=x1$residLCI, uy=x1$residUCI,
	prepanel=prepanel.ci, 
	panel=function(x,y,...){
		panel.superpose(x,y,...)
		panel.abline(h=c(-0.75, 0, 0.75), lty=2)
	}, 
	panel.groups=panel.ci,
        arrow.length = 0,
	type="p",
	auto.key=list(text=c("no chronic antibiotic","chronic antibiotic"),cex=1.2),
	cex=0.8, scales=list(cex=1.2),
	xlab=list(label="expected probability",cex=1.2),
	ylab=list(label="residual",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	stapleLength = 0.01)

## posterior distributions of latent residuals (Albert & Chib)

## Define functions for the truncated logistic distribution

## Inverse CDF
qlogis.trunc = function(p,location=0,scale=1,lower=-Inf,upper=Inf)
	qlogis(p*plogis(upper,location,scale)+(1-p)*plogis(lower,location,scale),location,scale)

## Generate random values
rlogis.trunc = function(n,location=0,scale=1,lower=-Inf,upper=Inf)
	qlogis.trunc(runif(n),location,scale,lower,upper)

## density
dlogis.trunc = function(x,location=0,scale=1,lower=-Inf,upper=Inf)
	ifelse((x < lower) | (x > upper),
		0, 
		dlogis(x, location, scale) / 
			(plogis(upper, location, scale) - plogis(lower, location, scale))
	)

## CDF
plogis.trunc = function(x,location=0,scale=1,lower=-Inf,upper=Inf)
	ifelse(x < lower,
		0,
		ifelse(x > upper,
			1, 
			(plogis(x, location, scale) - plogis(lower, location, scale)) / 
			(plogis(upper, location, scale) - plogis(lower, location, scale))
		)
	)


## Get the observed data
x1 = dataProb1
## Calculate the posterior mean of Pr(>= 1 pulmonary exacerbation)
x1$pExacExp = apply(pred, 2, mean)

## Calculate the posterior medians and 90% credible intervals of the residuals
residual = t(sapply(1:nrow(x1), 
	function(i, data, pred){
		if(data$exac[i] == 1)
			out = rlogis.trunc(nrow(pred), lower = -logit(pred[,i]))
		else
			out = rlogis.trunc(nrow(pred), upper = -logit(pred[,i]))
		quantile(out, probs=c(0.05,0.5,0.95))
	},
	data = x1, pred = pred))

dimnames(residual)[[2]] = c("residLCI", "residMed", "residUCI")

x1 = cbind(x1, residual)

## Plot the residuals versus the mean predicted Pr(>= 1 pulmonary exacerbation)
xyplot(residMed ~ pExacExp | as.factor(dose), data=x1, groups=AntiBio,
	ly=x1$residLCI, uy=x1$residUCI,
	prepanel=prepanel.ci, 
	panel=function(x,y,...){
		panel.superpose(x,y,...)
		panel.abline(h=c(qlogis(0.025), 0, qlogis(0.975)), lty=2)
	}, 
	panel.groups=panel.ci,
        arrow.length = 0,
	type="p",
	auto.key=list(text=c("no chronic antibiotic","chronic antibiotic"),cex=1.2),
	cex=0.8, scales=list(cex=1.2),
	xlab=list(label="expected probability",cex=1.2),
	ylab=list(label="residual",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	stapleLength = 0.01)

dev.off()

