## Template R script for WinBUGS execution
## Required edits by the user are indicated by question marks

## Fill in model name and name of directory containing script 
## and model files
modelName <- "handsOn1Base" # root names of model file
exampleDir <- "handsOn1"

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, exampleDir)
toolsDir <- file.path(courseDir, "bugsTools") ## change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)

## Add any additional R packages you need here
library(R2WinBUGS)
library(coda)
library(lattice)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## Get data files
## Change as needed to fit your problem
xdata = read.csv("ME2ExacerbationData.csv",as.is=T)

################################################################################################

## create WinBUGS data set

xdata2 <- aggregate(cbind(xdata[c("exac")], n = rep(1, nrow(xdata))), 
	list(dose = xdata$dose), sum)

bugsdata = list(
    nDoses = nrow(xdata2),
	dose = xdata2$dose,
	nexac = xdata2$exac,
	n = xdata2$n
)

## create initial estimates
bugsinit = function() list(
    theta = rnorm(2, c(-0.34, -0.015), 0.1)
)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("theta")

## Additional variables to monitor
otherRVs <- c("nexacPred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
## run WinBUGS

n.chains = 3
n.iter = 10000
n.burnin = 4000
n.thin = 5
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = FALSE, debug = TRUE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory

save.model(bugs.fit, modelName)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))
if(bugs.fit$isDIC){
	cat(paste(c("Edeviance =","pD =","DIC ="),c(mean(posterior[,"deviance"]), bugs.fit$pD, bugs.fit$DIC)),
		file=paste(modelName,"/",modelName,".deviance.txt",sep=""))
}

################################################################################################
## posterior predictive distributions

## Example based on MI255 Hands-on problem 1:
## Categorical dependent variable (exac) where Pr(exac = 1) is
## a function of dose

pred = posterior[,grep("nexacPred\\[",dimnames(posterior)[[2]])]

x1 <- xdata2
x1$type <- rep("observed",nrow(x1))
x2 <- rbind(x1,x1,x1)
x2$exac <- as.vector(t(apply(pred,2,quantile,probs=c(0.05,0.5,0.95))))
x2$type <- rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 <- rbind(x1,x2)

xyplot(exac~dose, x1,
	groups=type, panel=panel.superpose, type=c("l","l","l","p"), lty=c(2,2,1,0),
	col=c(2,2,4,1),
	pch=c(NA,NA,NA,19), lwd=2, cex=1, scales=list(cex=1),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="fraction of patients with pulmonary exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	ylim=c(0,1.1*max(x1$exac)),
	strip = function(...) strip.default(..., style = 1))

dev.off()	
