## example of count data
## fictional data: number of coughing events

modelName <- "handsOn4Poisson" # root names of model file
exampleDir <- "handsOn4"

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, exampleDir)
toolsDir <- file.path(courseDir, "bugsTools") ## change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## get data files
dataProb4 = read.csv("ME2CoughCountData.csv",as.is=T)

################################################################################################
## exploratory plots

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"EdaPlots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

countFreq = aggregate(1:nrow(dataProb4), list(dose=dataProb4$dose, coughCount=dataProb4$coughCount),
	length)
countFreq = merge(countFreq, expand.grid(dose=unique(dataProb4$dose), 
	coughCount=1:max(dataProb4$coughCount)), all=T)
countFreq = countFreq[order(countFreq$coughCount, countFreq$dose),]
countFreq$x = ifelse(is.na(countFreq$x), 0, countFreq$x)

barchart(x ~ coughCount, countFreq, groups=dose, horizontal=F, origin=0, stack=F, auto.key=T)
aggregate(dataProb4["coughCount"], list(dose=dataProb4$dose), mean)
aggregate(dataProb4["coughCount"], list(dose=dataProb4$dose), median)
aggregate(dataProb4["coughCount"], list(dose=dataProb4$dose), var)

barchart(x ~ coughCount | as.factor(dose), countFreq, horizontal=F, origin=0,
	panel=function(x,y,...){
		panel.barchart(x,y,...)
		xm1 = as.numeric(as.character(x)) - 1
		gmean = sum(xm1*y)/sum(y)
		gvar = sum((xm1-gmean)^2*y)/(sum(y)-1)
		gmean = signif(gmean,3)
		gvar = signif(gvar,3)
		panel.text(8,35,paste("mean =",gmean,
			"\nvariance =",gvar))
		panel.lines(1+(0:max(countFreq$coughCount)), sum(y)*dpois(0:max(countFreq$coughCount),gmean),
			col="black", lwd=2)
		panel.text(12, 20, paste("poisson(", gmean, ")",sep=""), adj=c(0,0.5))
		panel.arrows(12, 20, 6, sum(y)*dpois(5,gmean), lwd=2)
	},
	cex=1.2,scales=list(cex=1.2, x=list(at=seq(1,max(countFreq$coughCount)+2,by=2),
	labels=seq(0,max(countFreq$coughCount)+1,by=2))),
	xlab=list(label="count of cough reports by each patient",cex=1.2),
	ylab=list(label="number of patients",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	layout=c(1,4))

x1 = aggregate(dataProb4["coughCount"], list(dose=dataProb4$dose), mean)
x1$n = as.vector(tapply(dataProb4$coughCount,list(dose=dataProb4$dose),length))
x1$sd = as.vector(tapply(dataProb4$coughCount,list(dose=dataProb4$dose),sd))
x1$lci = x1$coughCount - 2*x1$sd/sqrt(x1$n)
x1$uci = x1$coughCount + 2*x1$sd/sqrt(x1$n)
xyplot(coughCount~dose,data=x1,
	ly=x1$lci,uy=x1$uci,
	prepanel=prepanel.ci,panel=panel.ci,
	type="b",
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="mean count of coughing events",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

dev.off()

################################################################################################

## create WinBUGS data set

bugsdata = list(
	nPatients = nrow(dataProb4),
	tEnd = dataProb4$time,
	dose = dataProb4$dose,
	nCough = dataProb4$coughCount
)

## create initial estimates
bugsinit = function() list(
	loga = rnorm(1,log(1.5/24),0.5),
	logb = rnorm(1,log(((2.88-1.5)/24)/60),0.5)
)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("a", "b")

## Additional variables to monitor
otherRVs <- c("lambda", "nCoughPred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
## run WinBUGS

n.chains = 3
n.iter = 10000
n.burnin = 1000
n.thin = 5
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = FALSE, debug = FALSE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory

save.model(bugs.fit, modelName, n.chains = n.chains)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))
if(bugs.fit$isDIC){
	cat(paste(c("Edeviance =","pD =","DIC ="),c(mean(posterior[,"deviance"]), bugs.fit$pD, bugs.fit$DIC)),
		file=paste(modelName,"/",modelName,".deviance.txt",sep=""))
}

################################################################################################
## posterior predictive distributions

pred = posterior[,grep("nCoughPred\\[",dimnames(posterior)[[2]])]

x1 = aggregate(dataProb4["coughCount"], list(dose=dataProb4$dose), mean)
x1$dose = as.numeric(as.character(x1$dose))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$coughCount = as.vector(t(sapply(tapply(1:nrow(dataProb4),
	list(dose=dataProb4$dose),
	function(i,pred) apply(pred[,i],1,mean),pred=pred),quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(coughCount~dose,x1,groups=type,
	type=c(rep("l",3),"p"),lty=c(2,2,1,0),
	col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
	lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose (mg/d)",cex=1.2),
	ylab=list(label="mean number of coughing events",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

x1 = aggregate(dataProb4["coughCount"], list(dose=dataProb4$dose), var)
x1$dose = as.numeric(as.character(x1$dose))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$coughCount = as.vector(t(sapply(tapply(1:nrow(dataProb4),
	list(dose=dataProb4$dose),
	function(i,pred) apply(pred[,i],1,var),pred=pred),quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(coughCount~dose,x1,groups=type,
	type=c(rep("l",3),"p"),lty=c(2,2,1,0),
	col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
	lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose (mg/d)",cex=1.2),
	ylab=list(label="variance of the number of coughing events",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))


x1 = tapply(1:nrow(dataProb4), list(dose=dataProb4$dose, 
	coughCount=factor(dataProb4$coughCount, levels=0:max(dataProb4$coughCount))),
	length)
x1 =  ifelse(is.na(x1), 0, x1)
x1 = data.frame(dose = rep(dimnames(x1)[[1]],ncol(x1)), 
	coughCount = rep(dimnames(x1)[[2]],ea=nrow(x1)),
	countFreq = as.vector(x1))
x1$type =rep("observed",nrow(x1))

x2 = rbind(x1,x1,x1)
x2$countFreq = as.vector(t(apply(apply(pred,1,function(pred,data){
		x1 = tapply(1:nrow(data), list(dose=data$dose, 
			coughCount=factor(pred, levels=0:max(data$coughCount))),
			length)
		as.vector(ifelse(is.na(x1), 0, x1))
	},data=dataProb4),1,quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

x1$coughCount = as.numeric(as.character(x1$coughCount))
x1$dose = as.numeric(as.character(x1$dose))
x1 = x1[order(x1$coughCount, x1$dose),]

xyplot(countFreq~coughCount|as.factor(dose),x1,groups=type,
	type=c(rep("l",3),"p"),lty=c(2,2,1,0),
	col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
	lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="number of coughing events",cex=1.2),
	ylab=list(label="number of patients",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))


dev.off()	
