## example of time-to-event data
## fictional pulmonary exacerbation data

modelName <- "handsOn5NoFEVTruncEmax" ## root names of model file
exampleDir <- "handsOn5"

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, exampleDir)
toolsDir <- file.path(courseDir, "bugsTools") ## change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
library(survival)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## get data files
dataProb5 = read.csv("ME3ExacerbationFEV1Data.csv",as.is=T)

################################################################################################
## create WinBUGS data set

bugsdata = list(
	nPatients = nrow(dataProb5),
	dose = dataProb5$dose,
	FEV10 = dataProb5$FEV1Baseline,
	tExac = ifelse(dataProb5$censored, NA, dataProb5$tExac),
	tCensor = ifelse(!dataProb5$censored, 0, dataProb5$tExac),
##	logFEV1 = log(dataProb5$FEV1),
##	logH0MeanPrior = log(0.029),
##	logH0PrecPrior = 1/(0.25^2),
##	betaMeanPrior = 0.031,
##	betaPrecPrior = 1/(0.015^2),
	logH0MeanPrior = log(0.029),
	logH0PrecPrior = 1.0E-4,
	betaMeanPrior = 0.031,
	betaPrecPrior = 1.0E-4,
	DStar = 30
)

## create initial estimates
bugsinit = function() list(
	logH0 = bugsdata$logH0MeanPrior,
	beta = bugsdata$betaMeanPrior,
##	logEStar = rnorm(1, log(10), 0.5),
##	logb = rnorm(1, log(2), 0.5)##,
	EStar = exp(rnorm(1, log(10), 0.5)),
	b = exp(rnorm(1, log(2), 0.5))##,
##	sigmaFEV1 = exp(rnorm(1, 0.2, 0.5))
)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("h0", "beta", "EStar", "b", "Emax", "ED50")

## Additional variables to monitor
otherRVs <- c("tExacPred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
## run WinBUGS

n.chains = 3
n.iter = 10000
n.burnin = 4000
n.thin = 5
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = FALSE, debug = TRUE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

##bugs.fit <- R2WinBUGS:::bugs.sims(parameters.to.save = parameters, n.chains = n.chains,
##                                  n.iter = n.iter, n.burnin = n.burnin, n.thin = n.thin)

## save scripts, data and results to a directory
save.model(bugs.fit, modelName, n.chains = n.chains)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
## posterior predictive distributions

doses = sort(unique(dataProb5$dose))

pred = posterior[,grep("tExacPred\\[",dimnames(posterior)[[2]])]

nPred = 120
tExacPred = seq(0, 12, len=nPred+1)[-(nPred+1)]
tExac = data.frame(dose = rep(doses, ea = nPred), 
	tExac = rep(tExacPred, length(unique(dataProb5$dose))),
	fNoExac = do.call(c, tapply(dataProb5$tExac, list(dose=dataProb5$dose), 
		function(x,xout){
			cdfFun = ecdf(x)
			1 - cdfFun(xout)
		},
		xout = tExacPred))
)
tExac$type = rep("observed", nrow(tExac))

x1 = rbind(tExac, tExac, tExac)
x1$fNoExac = as.vector(t(apply(
	apply(pred, 1,
		function(x, xout, dose){
			do.call(c, tapply(x, list(dose=dose), 
				function(x,xout){
					cdfFun = ecdf(x)
					1 - cdfFun(xout)
				},
				xout = xout))
		},
		xout = tExacPred, dose = dataProb5$dose),
	1, quantile, probs=c(0.05,0.5,0.95))))
x1$type = rep(c("5%ile","median","95%ile"), ea=nrow(tExac))
tExac = rbind(tExac, x1)

xyplot(fNoExac ~ tExac | as.factor(dose), tExac, groups = type,
	type="s",lty=c(2,2,1,1),
	col=c("blue","blue","red","black"),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="fraction of patients with no exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

dev.off()	
