## example of repeated time-to-event data
## fictional pulmonary exacerbation data

modelName <- "handsOn6NoRE" ## root names of model file
exampleDir <- "handsOn6"

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, exampleDir)
toolsDir <- file.path(courseDir, "bugsTools") ## change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
library(survival)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## get data files
dataProb6 = read.csv("ME2ExacerbationTimeData.csv",as.is=T)

################################################################################################

## create WinBUGS data set

dataProb6$tStart = c(0, dataProb6$tExac[-nrow(dataProb6)])
dataProb6$tStart = ifelse(dataProb6$tStart >= dataProb6$tExac, 0, dataProb6$tStart)
dataProb6$patID = as.numeric(as.factor(dataProb6$patient))  # sequential patient ID
data2 = dataProb6[!duplicated(dataProb6$patID),]  # one record per patient
data2 = data2[order(data2$patID),]  # assure data2 is ordered by patID

bugsdata = list(
	nObs = nrow(dataProb6),
	nPatients = nrow(data2),
	patient = dataProb6$patID,
	dose = data2$dose,
	FEV1Baseline = data2$FEV1Baseline,
	AntiBio = data2$AntiBio,
	tExac = ifelse(!dataProb6$censored, dataProb6$tExac - dataProb6$tStart, NA),
	tCensor = ifelse(!dataProb6$censored, 0, dataProb6$tExac - dataProb6$tStart),
	DStar = 60,
	nSim = 5
)

## create initial estimates

bugsinit = function() list(
	beta0 = rnorm(1, log(1.5/52), 0.2*abs(log(1.5/52))),
	betaFEV1 = rnorm(1, log(0.99), 0.2*abs(log(0.99))),
	betaAntiBio = rnorm(1, log(0.5), 0.2*abs(log(0.5))),
	EStar = exp(rnorm.trunc(1, log(0.75), 0.5, log(0.01), log(1))),
	b = 60 / exp(rnorm.trunc(1, log(30), 0.5, log(1), log(1000)))
)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("beta0", "betaFEV1", "betaAntiBio", "Emax", "ED50",
                      "EStar", "b")

## Additional variables to monitor
otherRVs <- c("tExacCond", "tExacPred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
## run WinBUGS

n.chains = 3
n.iter = 10000
n.burnin = 4000
n.thin = 5
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = FALSE, debug = TRUE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory

save.model(bugs.fit, modelName, n.chains = n.chains)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
## posterior predictive distributions

## prediction of future observations in the same subjects

doses = sort(unique(dataProb6$dose))

pred = posterior[,grep("tExacCond\\[",dimnames(posterior)[[2]])]

dataProb6$dtExac = dataProb6$tExac - dataProb6$tStart

fit = survfit(Surv(time = tStart, time2 = tExac, event = 1 - censored) ~ dose, dataProb6)
tExac = as.data.frame(summary(fit)[c("time","surv","strata")])
names(tExac) = c("tExac","fNoExac","dose")
levels(tExac$dose) = doses
tExac$dose = as.numeric(as.character(tExac$dose))
tExac$type = rep("observed", nrow(tExac))

nPred = 260
tExacPred = seq(0, 52, len=nPred+1)[-(nPred+1)]
x1 = data.frame(dose = rep(doses, ea = nPred), 
	tExac = rep(tExacPred, length(doses)))
x1 = rbind(x1, x1, x1)
x1$fNoExac = as.vector(t(apply(
	apply(pred, 1,
		function(x, xout, dose){
			do.call(c, tapply(x, list(dose=dose), 
				function(x,xout){
					cdfFun = ecdf(x)
					1 - cdfFun(xout)
				},
				xout = xout))
		},
		xout = tExacPred, dose = rep(data2$dose, ea=bugsdata$nSim)),
	1, quantile, probs=c(0.05,0.5,0.95))))
x1$type = rep(c("5%ile","median","95%ile"), ea=nrow(x1)/3)
tExac = rbind(tExac, x1)

xyplot(fNoExac ~ tExac | as.factor(dose), tExac, groups = type,
	type="s",lty=c(2,2,1,1),
	col=c("blue","blue","red","black"),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="fraction of patients with no exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

## prediction of future observations in new subjects

doses = sort(unique(dataProb6$dose))

pred = posterior[,grep("tExacPred\\[",dimnames(posterior)[[2]])]

dataProb6$dtExac = dataProb6$tExac - dataProb6$tStart

fit = survfit(Surv(time = tStart, time2 = tExac, event = 1 - censored) ~ dose, dataProb6)
tExac = as.data.frame(summary(fit)[c("time","surv","strata")])
names(tExac) = c("tExac","fNoExac","dose")
levels(tExac$dose) = doses
tExac$dose = as.numeric(as.character(tExac$dose))
tExac$type = rep("observed", nrow(tExac))

nPred = 260
tExacPred = seq(0, 52, len=nPred+1)[-(nPred+1)]
x1 = data.frame(dose = rep(doses, ea = nPred), 
	tExac = rep(tExacPred, length(doses)))
x1 = rbind(x1, x1, x1)
x1$fNoExac = as.vector(t(apply(
	apply(pred, 1,
		function(x, xout, dose){
			do.call(c, tapply(x, list(dose=dose), 
				function(x,xout){
					cdfFun = ecdf(x)
					1 - cdfFun(xout)
				},
				xout = xout))
		},
		xout = tExacPred, dose = rep(data2$dose, ea=bugsdata$nSim)),
	1, quantile, probs=c(0.05,0.5,0.95))))
x1$type = rep(c("5%ile","median","95%ile"), ea=nrow(x1)/3)
tExac = rbind(tExac, x1)

xyplot(fNoExac ~ tExac | as.factor(dose), tExac, groups = type,
	type="s",lty=c(2,2,1,1),
	col=c("blue","blue","red","black"),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="fraction of patients with no exacerbation",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

dev.off()	

